/********************************************************************
 * FileName:        output_mngt.c
 * Dependencies:    See INCLUDES section below
 * Processor:       MCP19111
 * Compiler:        XC8 v1.11
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement:
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the "Company") for its PICmicro Microcontroller is intended and
 * supplied to you, the Company's customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in vsiolation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *********************************************************************
 *
 * Version              Date        Comment
 * 1.000                05.12.2012  Initial release
 * 1.001                03.03.2014  Enabling low side when OFF to prevent floating output
 * 1.002                10.21.2014  Adding output calibration using ADC for MCP19118/9 device.
 ********************************************************************/

#include "output_mngt.h"

//Global variables

unsigned char m_TMR1H = V_TMR1H; //Variable allowing update soft start in dbg
unsigned char m_TMR1L = V_TMR1L; //Variable allowing update soft start in dbg
unsigned int m_voutADCTarget = VOUT_LINEAR; //For MCP19118/9 calibration
//Function declaration
void calibrate();
/********************************************************************
 * Function:        	OVRamp
 *
 * Overview:        	First, this function stop the timer 1.
 *                      If the soft start is NOT enabled, the funtion directy
 *                      set the ouput voltage and protection.
 *                      If the soft start is enabled, the funtion will increase
 *                      the output voltage level and set the correct timer 1
 *                      value for next interrupt. It will also re-enable
 *                      the timer1. This function is called at each timer 1
 *                      interrupt.
 *
 *                      When the output voltage level reaches its
 *                      desired level, timer 1 is not re-enabled.
 *                      It also enable output protections at the end of the ramp
 *******************************************************************/

void voutRampTarget(void) {
    TMR1ON = 0; //disabled timer
    if (OVCCON != m_OVCCON_target) //Output voltage is ramping?
    {
        ABECON = V_ABECON & NOT_OV_UV_MASK;
        TMR1H = m_TMR1H;
        TMR1L = m_TMR1L;
        TMR1ON = 1; //enabled timer
        if (OVCCON < 0x0A)
            PWMRL = 0x01;
    } else if (!HIDIS) //Is the output enabled? 
    {

#if (V_DEVICE_TARGET == DEV_MCP19118) || (V_DEVICE_TARGET == DEV_MCP19119)        
        calibrate(); //MCP19118/9 calibration
#endif
        ABECON = V_ABECON; //restore output protection
    }
    //Internal ramp for steps. Using timer for slow ramp (>10ms) or a while loop for fast ramp
    unsigned int TMR = (m_TMR1H << 8) | m_TMR1L;
    if (TMR < 0xFEE1)//TMR value is inverted... (means >10ms?)
    { //Slow ramp
        if (OVCCON > m_OVCCON_target) {
            OVCCON--;
            if (OVCCON < 0x0A)
                OVCCON = 0x00;
        }
        if (OVCCON < m_OVCCON_target)
            OVCCON++;

        if ((OVCCON > 0x0A) && (OVCCON < 0x1A))
            PWMRL = PWMRL++;
        else if (OVCCON > 0x1A)
            PWMRL = V_PWMRL;
    } else {//fast ramp
        unsigned int rampWaitTime = ((~TMR) >> 5);
        unsigned int i = 0;
        while (OVCCON != m_OVCCON_target) {
            if (OVCCON > m_OVCCON_target) {
                OVCCON--;
                if (OVCCON < 0x0A)
                    OVCCON = 0x00;
            }
            if (OVCCON < m_OVCCON_target)
                OVCCON++;

            if ((OVCCON > 0x0A) && (OVCCON < 0x1A))
                PWMRL = PWMRL++;
            else if (OVCCON > 0x1A)
                PWMRL = V_PWMRL;
            for (i = 0; i < rampWaitTime; i++);
        }
    }

    if (OVCCON > 0) {
        T2CON = V_T2CON; //enable timer.
        HIDIS = 0; //HIDRV, 1=disable, 0=enable
        PUEN = 0; //+Vsen pull-up, 1=enable, 0=disable
        

    } else {
        T2CON = V_T2CON & 0xFB; //disable timer.
        PWMRL = 0x00;
        HIDIS = 1; //HIDRV, 1=disable, 0=enable
        PUEN = 1; //+Vsen pull-up, 1=enable, 0=disable
    }
}

/********************************************************************
 * Function:        	switchOffOutput
 *
 * Overview:        	Stop switching without soft stop and disable output
 *                      protection
 *******************************************************************/
void switchOffOutput(void) {
    softStopOutput();
    ABECON = V_ABECON & NOT_OV_UV_MASK;
    PWMRL = 0x00;
    HIDIS = 1; //HIDRV, 1=disable, 0=enable
    PUEN = 1; //+Vsen pull-up, 1=enable, 0=disable
    OVCCON = 0;
}

/********************************************************************
 * Function:        	restoreOutput
 *
 * Overview:        	Set the output voltage:
 *                      -With soft start if enable
 *                      -Set output voltage from the GUI at first use
 *                      -Restore last output voltage if previously stopped
 *******************************************************************/
void restoreOutput(void) {
    m_OVCCON_target = V_OVCCON;
    OPERATION[0] |= OPERATION_ENABLE;
    TMR1ON = 1;

    T2CON = V_T2CON; //enable timer.
    HIDIS = 0; //HIDRV, 1=disable, 0=enable
    PUEN = 0; //+Vsen pull-up, 1=enable, 0=disable
}

/********************************************************************
 * Function:        	softStopOutput
 *
 * Overview:        	Disabled the output voltage with soft-stop:
 *                      -Store the current output voltage in case of
 *                       future restoration.
 *                      -Disable the output protection
 *                      -Set the new new output target to 0 and enabled Timer1
 *******************************************************************/
void softStopOutput(void) {
    OVFCON &= 0x80;
    m_OVCCON_target = m_voutADCTarget; //Dummy read to avoid compiler optimisation on voutADCTarget
    m_OVCCON_target = 0;
    TMR1ON = 1;
    OPERATION[0] &= ~OPERATION_ENABLE;
}

/********************************************************************
 * Function:        	checkEnPin
 *
 * Overview:        	Enable or not the output in function of the pin
 *                      stat and the device configuration.
 *******************************************************************/
unsigned char EN_PIN_old = 1;

void checkEnPin(void) {
    if (((*ON_OFF_CONFIG) & ON_OFF_PIN_ON) && ((*ON_OFF_CONFIG) & ON_OFF_ALWAYS_ON_B)) {
        //rising edge detection
        if (EN_PIN == 1 && EN_PIN_old == 0) {
            if (OPERATION[0] & OPERATION_ENABLE)
                softStopOutput();
            else
                restoreOutput();
        }
        EN_PIN_old = EN_PIN;
    }
}

/*******************************************************************************
 * Calibrate routine for the MCP19119 evaluation board.
 *
 * This routine cycles through the output voltage options and uses the ADC to
 *  calibrate the coarse and fine settings.
 ******************************************************************************/
void calibrate() {
    unsigned char i;
    unsigned int acc = 0;
    unsigned long tmp;
    signed char corrDirection = 0;

    TMR0 = 0x00; //Clear timer 0
    while (TMR0 < DLY_SS); //Wait until we've reached the number of ticks
    SET_ADC_CHANEL(ADC_INT_VREG); //Set channel

    while (1) { //Iterate until the reading matches the setting
        TMR0 = 0x00; //Clear timer 0
        while (TMR0 < DLY_SS); //Wait until we've reached the number of ticks
        GO_nDONE = 1;
        NOP();
        while (GO_nDONE); //Ignore the first reading, letting vout settle
        acc = 0; //Clear the accumulator
        for (i = 16; i > 0; i--) {
            GO_nDONE = 1;
            NOP();
            while (GO_nDONE); //Start the ADC and wait for completion
            acc += ADRES; //Accumulate the reading
        } //for (j = 16; j > 0; j--)
        acc >>= 2; //Divide by 4 to average
        acc -= calib.D2VOO; //Substract out the offset
        tmp = (unsigned long) calib.D2VOG*acc; //Multiply by the gain correction
        acc = (unsigned int) ((tmp & 0xFFFF00) >> 8);
        acc >>= 3; //Roll right to correct bit position

        //If the correction direction changes, we are done.
        
        if (acc < m_voutADCTarget) //Rising correction
        {
            if (corrDirection == -1)
                break;
            corrDirection=1;
            if ((m_voutADCTarget - acc) > 40) { //Adjust the coarse/fine settings
                OVCCON ++; //Take a coarse step
            } else if ((OVFCON & 0x1F) == 0x1F)  { //Check for fine control overflow
                OVFCON = 0x80;
                OVCCON++;
            } else {
                OVFCON++;
            } //if

        } else { //Falling correction
            if (corrDirection == 1)
                break;
            corrDirection=-1;
            if ((acc - m_voutADCTarget) > 40) { //Adjust the coarse/fine settings
                OVCCON --; //Take a coarse step
            } else if ((OVFCON & 0x1F) == 0x00)  { //Check for fine control overflow
                OVFCON = 0x80 | 0x1F;
                OVCCON--;
            } else {
                OVFCON--;
            } //if
        }
    } //while
} //calibrate()